/* Handle the printing of info to the screen */


#include "angband.h"

/*
 * Print character info at given row, column in a 13 char field
 */
static void prt_field(cptr info, int row, int col)
{
	/* Dump 13 spaces to clear */
	c_put_str(TERM_WHITE, "             ", row, col);

	/* Dump the info itself */
	c_put_str(TERM_L_BLUE, info, row, col);
}


/*
 * Converts stat num into a six-char (right justified) string
 */
void cnv_stat(int val, char *out_val)
{
	if (val > 18)
	{
		int bonus = (val - 18);

		if (bonus >= 220)
		{
			sprintf(out_val, "18/%3s", "***");
		}
		else if (bonus >= 100)
		{
			sprintf(out_val, "18/%03d", bonus);
		}
		else
		{
			sprintf(out_val, " 18/%02d", bonus);
		}
	}
	else
	{
		sprintf(out_val, "    %2d", val);
	}
}


/*
 * Print character stat in given row, column
 */
void prt_stat(int stat, int max, int cur)
{
	char tmp[32];

	if (cur < max)
	{
		Term_putstr(0, ROW_STAT + stat, -1, TERM_WHITE, stat_names_reduced[stat]);
		cnv_stat(cur, tmp);
		Term_putstr(COL_STAT + 6, ROW_STAT + stat, -1, TERM_YELLOW, tmp);
	}

	else
	{
		Term_putstr(0, ROW_STAT + stat, -1, TERM_WHITE, stat_names[stat]);
		cnv_stat(cur, tmp);
		Term_putstr(COL_STAT + 6, ROW_STAT + stat, -1, TERM_L_GREEN, tmp);
	}
}

/*
 * Prints "title", including "wizard" or "winner" as needed.
 */
void prt_title(cptr title)
{
	prt_field(title, ROW_TITLE, COL_TITLE);
}


/*
 * Prints level and experience
 */
void prt_level(int level, int max, int cur, int adv)
{
	char tmp[32];

	sprintf(tmp, "%6d", level);

	Term_putstr(0, ROW_LEVEL, -1, TERM_WHITE, "LEVEL ");
	Term_putstr(COL_LEVEL + 6, ROW_LEVEL, -1, TERM_L_GREEN, tmp);

	sprintf(tmp, "%8ld", (long)cur);

	if (cur >= max)
	{
		Term_putstr(0, ROW_EXP, -1, TERM_WHITE, "EXP ");
		Term_putstr(COL_EXP + 4, ROW_EXP, -1, TERM_L_GREEN, tmp);
	}
	else
	{
		Term_putstr(0, ROW_EXP, -1, TERM_WHITE, "Exp ");
		Term_putstr(COL_EXP + 4, ROW_EXP, -1, TERM_YELLOW, tmp);
	}
}

/*
 * Prints current gold
 */
void prt_gold(int gold)
{
	char tmp[32];

	put_str("AU ", ROW_GOLD, COL_GOLD);
	sprintf(tmp, "%9ld", (long)gold);
	c_put_str(TERM_L_GREEN, tmp, ROW_GOLD, COL_GOLD + 3);
}

/*
 * Prints current AC
 */
void prt_ac(int ac)
{
	char tmp[32];

	put_str("Cur AC ", ROW_AC, COL_AC);
	sprintf(tmp, "%5d", ac);
	c_put_str(TERM_L_GREEN, tmp, ROW_AC, COL_AC + 7);
}

/*
 * Prints Max/Cur hit points
 */
void prt_hp(int max, int cur)
{
	char tmp[32];
	byte color;

	put_str("Max HP ", ROW_MAXHP, COL_MAXHP);

	sprintf(tmp, "%5d", max);
	color = TERM_L_GREEN;

	c_put_str(color, tmp, ROW_MAXHP, COL_MAXHP + 7);


	put_str("Cur HP ", ROW_CURHP, COL_CURHP);

	sprintf(tmp, "%5d", cur);

	if (cur >= max)
	{
		color = TERM_L_GREEN;
	}
	else if (cur > max / 10)
	{
		color = TERM_YELLOW;
	}
	else
	{
		color = TERM_RED;
	}

	c_put_str(color, tmp, ROW_CURHP, COL_CURHP + 7);
}

/*
 * Prints Max/Cur spell points
 */
void prt_sp(int max, int cur)
{
	char tmp[32];
	byte color;

	put_str("Max SP ", ROW_MAXSP, COL_MAXSP);
 
	sprintf(tmp, "%5d", max);
	color = TERM_L_GREEN;

	c_put_str(color, tmp, ROW_MAXSP, COL_MAXSP + 7);


	put_str("Cur SP ", ROW_CURSP, COL_CURSP);

	sprintf(tmp, "%5d", cur);

	if (cur >= max)
	{
		color = TERM_L_GREEN;
	}
	else if (cur > max / 10)
	{
		color = TERM_YELLOW;
	}
	else
	{
		color = TERM_RED;
	}

	c_put_str(color, tmp, ROW_CURSP, COL_CURSP + 7);
}

/*
 * Prints depth into the dungeon
 */
void prt_depth(int depth)
{
	char depths[32];

	if (!depth)
	{
		(void)strcpy(depths, "Town");
	}
	else if (depth_in_feet)
	{
		(void)sprintf(depths, "%d ft", depth * 50);
	}
	else
	{
		(void)sprintf(depths, "Lev %d", depth);
	}

	/* Right-Adjust the "depth" and clear old values */
	prt(format("%7s", depths), 23, COL_DEPTH);
}

/*
 * Prints hunger information
 */
void prt_hunger(int food)
{
	if (food < PY_FOOD_FAINT)
	{
		c_put_str(TERM_RED, "Weak  ", ROW_HUNGRY, COL_HUNGRY);
	}

	else if (food < PY_FOOD_WEAK)
	{
		c_put_str(TERM_ORANGE, "Weak  ", ROW_HUNGRY, COL_HUNGRY);
	}

	else if (food < PY_FOOD_ALERT)
	{
		c_put_str(TERM_YELLOW, "Hungry", ROW_HUNGRY, COL_HUNGRY);
	}

	else if (food < PY_FOOD_FULL)
	{
		c_put_str(TERM_L_GREEN, "      ", ROW_HUNGRY, COL_HUNGRY);
	}

	else if (food < PY_FOOD_MAX)
	{
		c_put_str(TERM_L_GREEN, "Full  ", ROW_HUNGRY, COL_HUNGRY);
	}

	else
	{
		c_put_str(TERM_GREEN, "Gorged", ROW_HUNGRY, COL_HUNGRY);
	}
}

/*
 * Prints blindness status
 */
void prt_blind(bool blind)
{
	if (blind)
	{
		c_put_str(TERM_ORANGE, "Blind", ROW_BLIND, COL_BLIND);
	}
	else
	{
		put_str("     ", ROW_BLIND, COL_BLIND);
	}
}

/*
 * Prints confused status
 */
void prt_confused(bool confused)
{
	if (confused)
	{
		c_put_str(TERM_ORANGE, "Confused", ROW_CONFUSED, COL_CONFUSED);
	}
	else
	{
		put_str("        ", ROW_CONFUSED, COL_CONFUSED);
	}
}

/*
 * Prints fear status
 */
void prt_afraid(bool fear)
{
	if (fear)
	{
		c_put_str(TERM_ORANGE, "Afraid", ROW_AFRAID, COL_AFRAID);
	}
	else
	{
		put_str("      ", ROW_AFRAID, COL_AFRAID);
	}
}

/*
 * Prints poisoned status
 */
void prt_poisoned(bool poisoned)
{
	if (poisoned)
	{
		c_put_str(TERM_ORANGE, "Poisoned", ROW_POISONED, COL_POISONED);
	}
	else
	{
		put_str("        ", ROW_POISONED, COL_POISONED);
	}
}

/*
 * Prints paralyzed/searching status
 */
void prt_state(bool paralyzed, bool searching, bool resting)
{
	byte attr = TERM_WHITE;

	char text[16];

	if (paralyzed)
	{
		attr = TERM_RED;

		strcpy(text, "Paralyzed!");
	}

	else if (searching)
	{
		if (player.pclass != CLASS_ROGUE)
		{
			strcpy(text, "Searching ");			
		}
		
		else
		{
			attr = TERM_L_DARK;
			strcpy(text,"Stlth Mode");
		}
		
	}

	else if (resting)
	{
		strcpy(text, "Resting   ");
	}
	else
	{
		strcpy(text, "            ");
	}

	c_put_str(attr, text, ROW_STATE, COL_STATE);
}

/*
 * Prints speed
 */
void prt_speed(int speed)
{
	int attr = TERM_WHITE;
	char buf[32] = "";

	p_ptr->pspeed = speed;
	if (speed > 0)
	{
		attr = TERM_L_GREEN;
		sprintf(buf, "Fast (+%d)", speed);
	}

	else if (speed < 0)
	{
		attr = TERM_L_UMBER;
		sprintf(buf, "Slow (%d)", speed);
	}

	/* Display the speed */
	c_put_str(attr, format("%-14s", buf), ROW_SPEED, COL_SPEED);
}

/*
 * Prints ability to gain spells
 */
void prt_study(bool study)
{
	if (study)
	{
		put_str("Study", ROW_STUDY, 64);
	}
	else
	{
		put_str("     ", ROW_STUDY, COL_STUDY);
	}
}

/*
 * Prints cut status
 */
void prt_cut(int cut)
{
	if (cut > 1000)
	{
		c_put_str(TERM_L_RED, "Mortal wound", ROW_CUT, COL_CUT);
	}
	else if (cut > 200)
	{
		c_put_str(TERM_RED, "Deep gash  ", ROW_CUT, COL_CUT);
	}
	else if (cut > 100)
	{
		c_put_str(TERM_RED, "Severe cut  ", ROW_CUT, COL_CUT);
	}
	else if (cut > 50)
	{
		c_put_str(TERM_ORANGE, "Nasty cut   ", ROW_CUT, COL_CUT);
	}
	else if (cut > 25)
	{
		c_put_str(TERM_ORANGE, "Bad cut     ", ROW_CUT, COL_CUT);
	}
	else if (cut > 10)
	{
		c_put_str(TERM_YELLOW, "Light cut  ", ROW_CUT, COL_CUT);
	}
	else if (cut)
	{
		c_put_str(TERM_YELLOW, "Graze     ", ROW_CUT, COL_CUT);
	}
	else
	{
		put_str("            ", ROW_CUT, COL_CUT);
	}
}

/*
 * Prints stun status
 */
void prt_stun(int stun)
{
	if (stun > 100)
	{
		c_put_str(TERM_RED, "Knocked out ", ROW_STUN, COL_STUN);
	}
	else if (stun > 50)
	{
		c_put_str(TERM_ORANGE, "Heavy stun  ", ROW_STUN, COL_STUN);
	}
	else if (stun)
	{
		c_put_str(TERM_ORANGE, "Stun        ", ROW_STUN, COL_STUN);
	}
	else
	{
		put_str("            ", ROW_STUN, COL_STUN);
	}
}

/*
 * Prints race/class info
 */
void prt_basic(void)
{
	cptr r, c;

	switch(race)
	{
		case 0: r = "Human"; break;
		case 1: r = "Half-elf"; break;
		case 2: r = "Elf"; break;
		case 3: r = "Hobbit"; break;
		case 4: r = "Gnome"; break;
		case 5: r = "Dwarf"; break;
		case 6: r = "Half-orc"; break;
		case 7: r = "Half-troll"; break;
		case 8: r = "Dunadan"; break;
		case 9: r = "High-elf"; break;
	}

	switch(class)
	{
		case 0: c = "Warrior"; break;
		case 1: c = "Mage"; break;
		case 2: c = "Priest"; break;
		case 3: c = "Rogue"; break;
		case 4: c = "Ranger"; break;
		case 5: c = "Paladin"; break;
	}

	prt_field(r, ROW_RACE, COL_RACE);
	prt_field(c, ROW_CLASS, COL_CLASS);
}

/*
 * Redraw the lag bar
 */

void prt_lag(u32b mark, u32b num)
{
	int attr=0;
	static int flipper =0;
	static u32b last=0;

	/* Default to "unknown" */

	num/=10;
	if(num>30000) return; // sometimes we catch a negative flip, just ignore it.


	Term_erase(COL_LAG, ROW_LAG, 12);
	/* Term_putstr(COL_LAG, ROW_LAG, 12, (flipper%2)?TERM_WHITE:TERM_L_DARK, "LAG: [-----]"); */

	Term_putstr(COL_LAG, ROW_LAG, 12, TERM_L_DARK, "LAG:[------]"); 

	if( mark == 999999999L ) {
		c_msg_print(format("Time out Mark: %lu, Last: %lu", mark,last));
		if( num < last ) {
			num=last;
		} else {
			last=num;
		};
		attr=TERM_VIOLET;
	} 

	// if(!(++flipper%1)) c_msg_print(format("Ticks %lu Mark %lu, Last: %lu, New: %lu, Latency %lu.", ticks,mark,last,num,(num+last)/2L));

	num=last=(num+last)/2L;
		
	num /= (500/6);
	if(!attr) {
		attr=TERM_L_GREEN;
		if(num > 3L) attr=TERM_YELLOW;
		if(num > 5L) attr=TERM_RED;
	};
	Term_putstr(COL_LAG + 5, ROW_LAG, num, attr, "******");
}

/*
 * Redraw the monster health bar
 */

void health_redraw(int num, byte attr)
{
	/* Not tracking */
	if (!attr)
	{
		/* Erase the health bar */
		Term_erase(COL_INFO, ROW_INFO, 12);
	}

	/* Tracking a monster */
	else
	{
		/* Default to "unknown" */
		Term_putstr(COL_INFO, ROW_INFO, 12, TERM_WHITE, "[----------]");

		/* Dump the current "health" (use '*' symbols) */
		Term_putstr(COL_INFO + 1, ROW_INFO, num, attr, "**********");
	}
}

/*
 * Choice window "shadow" of the "show_inven()" function. 
 */
static void display_inven(void)
{
	int	i, n, z = 0;
	int	wgt;

	object_type *o_ptr;

	char	o_name[80];

	char	tmp_val[80];

	/* Find the "final" slot */
	for (i = 0; i < INVEN_PACK; i++)
	{
		o_ptr = &inventory[i];

		/* Track non-empty slots */
		if (o_ptr->tval) z = i + 1;
	}

	/* Display the inventory */
	for (i = 0; i < z; i++)
	{
		o_ptr = &inventory[i];

		/* Start with an empty "index" */
		tmp_val[0] = tmp_val[1] = tmp_val[2] = ' ';

		/* Is this item acceptable? */
		if (item_tester_okay(o_ptr))
		{
			/* Prepare an "index" */
			tmp_val[0] = index_to_label(i);

			/* Bracket the "index" --(-- */
			tmp_val[1] = ')';
		}

		/* Display the index (or blank space) */
		Term_putstr(0, i, 3, TERM_WHITE, tmp_val);
		
		/* Describe the object */
		strcpy(o_name, inventory_name[i]);

		/* Obtain length of description */
		n = strlen(o_name);

		/* Clear the line with the (possibly indented) index */
		Term_putstr(3, i, n, o_ptr->sval, o_name);

		/* Erase the rest of the line */
		Term_erase(3+n, i, 255);

		/* Display the weight if needed */
		if (show_weights && o_ptr->weight)
		{
			wgt = o_ptr->weight;
			(void)sprintf(tmp_val, "%3d.%1d lb", wgt / 10, wgt % 10);
			Term_putstr(71, i, -1, TERM_WHITE, tmp_val);
		}
	}

	/* Erase the rest of the window */
	for (i = z; i < Term->hgt; i++)
	{
		/* Erase the line */
		Term_erase(0, i, 255);
	}
}


/*
 * Choice window "shadow" of the "show_equip()" function. 
 */
static void display_equip(void)
{
	int	i, n;
	int	wgt;

	object_type *o_ptr;

	char	o_name[80];

	char	tmp_val[80];

	/* Find the "final" slot */
	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)
	{
		o_ptr = &inventory[i];

		/* Start with an empty "index" */
		tmp_val[0] = tmp_val[1] = tmp_val[2] = ' ';

		/* Is this item acceptable? */
		if (item_tester_okay(o_ptr))
		{
			/* Prepare an "index" */
			tmp_val[0] = index_to_label(i);

			/* Bracket the "index" --(-- */
			tmp_val[1] = ')';
		}

		/* Display the index (or blank space) */
		Term_putstr(0, i - INVEN_WIELD, 3, TERM_WHITE, tmp_val);
		
		/* Describe the object */
		strcpy(o_name, inventory_name[i]);

		/* Obtain length of the description */
		n = strlen(o_name);

		/* Clear the line with the (possibly indented) index */
		Term_putstr(3, i - INVEN_WIELD, n, o_ptr->sval, o_name);

		/* Erase the rest of the line */
		Term_erase(3+n, i - INVEN_WIELD, 255);

		/* Display the weight if needed */
		if (show_weights && o_ptr->weight)
		{
			wgt = o_ptr->weight;
			(void)sprintf(tmp_val, "%3d.%1d lb", wgt / 10, wgt % 10);
			Term_putstr(71, i - INVEN_WIELD, -1, TERM_WHITE, tmp_val);
		}
	}

	/* Erase the rest of the window */
	for (i = INVEN_TOTAL - INVEN_WIELD; i < Term->hgt; i++)
	{
		/* Erase the line */
		Term_erase(0, i, 255);
	}
}


/*
 * Display the inventory.
 *
 * Hack -- do not display "trailing" empty slots
 */
void show_inven(void)
{
	int	i, j, k, l, z = 0;
	int	col, len, lim, wgt;

	object_type *o_ptr;

	char	o_name[80];

	char	tmp_val[80];

	int	out_index[23];
	byte	out_color[23];
	char	out_desc[23][80];


	/* Starting column */
	col = command_gap;

	/* Default "max-length" */
	len = 79 - col;

	/* Maximum space allowed for descriptions */
	lim = 79 - 3;

	/* Require space for weight (if needed) */
	lim -= 9;


	/* Find the "final" slot */
	for (i = 0; i < INVEN_PACK; i++)
	{
		o_ptr = &inventory[i];

		/* Track non-empty slots */
		if (o_ptr->tval) z = i + 1;
	}

	/* Display the inventory */
	for (k = 0, i = 0; i < z; i++)
	{
		o_ptr = &inventory[i];

		/* Is this item acceptable? */
		if (!item_tester_okay(o_ptr)) continue;

		/* Describe the object */
		strcpy(o_name, inventory_name[i]);

		/* Hack -- enforce max length */
		o_name[lim] = '\0';

		/* Save the object index, color, and descrtiption */
		out_index[k] = i;
		out_color[k] = o_ptr->sval;
		(void)strcpy(out_desc[k], o_name);

		/* Find the predicted "line length" */
		l = strlen(out_desc[k]) + 5;

		/* Be sure to account for the weight */
		l += 9;

		/* Maintain the maximum length */
		if (l > len) len = l;

		/* Advance to next "line" */
		k++;
	}

	/* Find the column to start in */
	col = (len > 76) ? 0 : (79 - len);

	/* Output each entry */
	for (j = 0; j < k; j++)
	{
		/* Get the index */
		i = out_index[j];

		/* Get the item */
		o_ptr = &inventory[i];

		/* Clear the line */
		prt("", j + 1, col ? col - 2 : col);

		/* Prepare and index --(-- */
		sprintf(tmp_val, "%c)", index_to_label(i));

		/* Clear the line with the (possibly indented) index */
		put_str(tmp_val, j + 1, col);

		/* Display the entry itself */
		c_put_str(out_color[j], out_desc[j], j + 1, col + 3);

		/* Display the weight if needed */
		if (show_weights && o_ptr->weight)
		{
			wgt = o_ptr->weight;
			(void)sprintf(tmp_val, "%3d.%1d lb", wgt / 10, wgt % 10);
			put_str(tmp_val, j + 1, 71);
		}
	}

	/* Make a "shadow" below the list (only if needed) */
	if (j && (j < 23)) prt("", j + 1, col ? col - 2 : col);

	/* Save the new column */
	command_gap = col;
}


/*
 * Display the equipment.
 */
void show_equip(void)
{
	int	i, j, k, l;
	int	col, len, lim, wgt;

	object_type *o_ptr;

	char	o_name[80];

	char	tmp_val[80];

	int	out_index[23];
	byte	out_color[23];
	char	out_desc[23][80];


	/* Starting column */
	col = command_gap;

	/* Default "max-length" */
	len = 79 - col;

	/* Maximum space allowed for descriptions */
	lim = 79 - 3;

	/* Require space for weight (if needed) */
	lim -= 9;


	/* Scan the equipment list */
	for (k = 0, i = INVEN_WIELD; i < INVEN_TOTAL; i++)
	{
		o_ptr = &inventory[i];

		/* Is this item acceptable? */
		if (!item_tester_okay(o_ptr)) continue;

		/* Describe the object */
		strcpy(o_name, inventory_name[i]);

		/* Hack -- enforce max length */
		o_name[lim] = '\0';

		/* Save the object index, color, and descrtiption */
		out_index[k] = i;
		out_color[k] = o_ptr->sval;
		(void)strcpy(out_desc[k], o_name);

		/* Find the predicted "line length" */
		l = strlen(out_desc[k]) + 5;

		/* Be sure to account for the weight */
		l += 9;

		/* Maintain the maximum length */
		if (l > len) len = l;

		/* Advance to next "line" */
		k++;
	}

	/* Find the column to start in */
	col = (len > 76) ? 0 : (79 - len);

	/* Output each entry */
	for (j = 0; j < k; j++)
	{
		/* Get the index */
		i = out_index[j];

		/* Get the item */
		o_ptr = &inventory[i];

		/* Clear the line */
		prt("", j + 1, col ? col - 2 : col);

		/* Prepare and index --(-- */
		sprintf(tmp_val, "%c)", index_to_label(i));

		/* Clear the line with the (possibly indented) index */
		put_str(tmp_val, j + 1, col);

		/* Display the entry itself */
		c_put_str(out_color[j], out_desc[j], j + 1, col + 3);

		/* Display the weight if needed */
		if (show_weights && o_ptr->weight)
		{
			wgt = o_ptr->weight * o_ptr->number;
			(void)sprintf(tmp_val, "%3d.%1d lb", wgt / 10, wgt % 10);
			put_str(tmp_val, j + 1, 71);
		}
	}

	/* Make a "shadow" below the list (only if needed) */
	if (j && (j < 23)) prt("", j + 1, col ? col - 2 : col);

	/* Save the new column */
	command_gap = col;
}


/*
 * Display inventory in sub-windows
 */
void fix_inven(void)
{
	int j;

	/* Scan windows */
	for (j = 0; j < 8; j++)
	{
		term *old = Term;

		/* No window */
		if (!ang_term[j]) continue;

		/* No relevant flags */
		if (!(window_flag[j] & PW_INVEN)) continue;

		/* Activate */
		Term_activate(ang_term[j]);

		/* Display inventory */
		display_inven();

		/* Fresh */
		Term_fresh();

		/* Restore */
		Term_activate(old);
	}
}


/*
 * Display equipment in sub-windows
 */
void fix_equip(void)
{
	int j;

	/* Scan windows */
	for (j = 0; j < 8; j++)
	{
		term *old = Term;

		/* No window */
		if (!ang_term[j]) continue;

		/* No relevant flags */
		if (!(window_flag[j] & PW_EQUIP)) continue;

		/* Activate */
		Term_activate(ang_term[j]);

		/* Display inventory */
		display_equip();

		/* Fresh */
		Term_fresh();

		/* Restore */
		Term_activate(old);
	}
}


/*
 * Display character sheet in sub-windows
 */
void fix_player(void)
{
	int j;

	/* Scan windows */
	for (j = 0; j < 8; j++)
	{
		term *old = Term;

		/* No window */
		if (!ang_term[j]) continue;

		/* No relevant flags */
		if (!(window_flag[j] & PW_PLAYER)) continue;

		/* Activate */
		Term_activate(ang_term[j]);

		/* Display inventory */
		display_player(FALSE);

		/* Fresh */
		Term_fresh();

		/* Restore */
		Term_activate(old);
	}
}

 
/*
 * Hack -- display recent messages in sub-windows
 *
 * XXX XXX XXX Adjust for width and split messages
 */

// [grk] Gross hack to display messages in seperate term

#define PMSG_TERM 4

void fix_message(void)
{
        int j, c, i,ii, pmdone, mdone;
        int w, h, pmw, pmh;
        int x, y;
        term *oldt;
	char from_us[30];

	/* Determine what messages from us are prefixed with */
	sprintf(from_us,"[%s]",nick);

        /* Scan windows */
        for (j = 0; j < 8; j++)
        {
                term *old = Term;

                /* No window */
                if (!ang_term[j]) continue;

                /* No relevant flags */
                if (!(window_flag[j] & PW_MESSAGE)) continue;

                /* Activate */
                Term_activate(ang_term[j]);

                /* Get size */
                Term_get_size(&w, &h);

                /* Dump messages */
                i=0; c=0;
                while(i<h)
                {
			byte a;
			cptr msg;

			msg = message_str(c++);

			if (ang_term[PMSG_TERM]) {

				if (msg[0] == '[') continue;
				else if(strstr(msg, "has entered the game")!=NULL) continue;
				else if(strstr(msg, "has left the game")!=NULL) continue;
				else if(strstr(msg, "committed suicide.")!=NULL) continue;
				else if(strstr(msg, "was killed by")!=NULL) continue;
				else if(strstr(msg, "was slain by")!=NULL) continue;
				else if(strstr(msg, "rises from the dead")!=NULL) continue;
				else if(strstr(msg, "ghost was destroyed by")!=NULL) continue;

			} 

			a = TERM_WHITE;

			if(msg[0] == '['){
				a = TERM_L_BLUE;
				if( (strstr(msg, nick)!=NULL) && (strstr(msg, from_us)==NULL) ) a = TERM_L_GREEN;
			} else {
				a=TERM_WHITE;

				if( (strstr(msg, "has entered the game")!=NULL)) a=TERM_L_DARK;
				if( (strstr(msg, "has left the game")!=NULL)) a=TERM_L_DARK;
				if( (strstr(msg, "committed suicide")!=NULL)) a=TERM_L_DARK;
				if( (strstr(msg, "ghost was destroyed by")!=NULL)) a=TERM_RED; 
				if( (strstr(msg, "was slain by")!=NULL)) a=TERM_YELLOW;
				if( (strstr(msg, "rises from the dead")!=NULL)) a=TERM_ORANGE;
				if( (strstr(msg, "was killed by")!=NULL)) a=TERM_RED;
			};

			/* Dump the message on the appropriate line */
			Term_putstr(0, (h - 1) - i, -1, a, msg);

			/* Cursor */
			Term_locate(&x, &y);

			/* Clear to end of line */
			Term_erase(x, y, 255);
		
			i++;
                }

                /* Fresh */
                Term_fresh();

                /* Restore */
                Term_activate(old);
        }

        // Display player messages in term 4
                
	oldt = Term;

	/* No window */
	if (ang_term[PMSG_TERM]) {

		/* Activate */
		Term_activate(ang_term[PMSG_TERM]);

		/* Get size */
		Term_get_size(&w, &h);

		/* Dump messages */
		i=0; c=0;
		while(i<h) {
			byte a;
			cptr msg;

			msg = message_str(c++);

			if(msg[0] == 0) { i++; continue; };

			if(msg[0] == '['){
				a = TERM_L_BLUE;
				if( (strstr(msg, nick)!=NULL) && (strstr(msg, from_us)==NULL) ) a = TERM_L_GREEN;
			} else {
				a=TERM_WHITE;

				if( (strstr(msg, "has entered the game")!=NULL)) a=TERM_L_DARK;
				if( (strstr(msg, "has left the game")!=NULL)) a=TERM_L_DARK;
				if( (strstr(msg, "committed suicide")!=NULL)) a=TERM_L_DARK;
				if( (strstr(msg, "ghost was destroyed by")!=NULL)) a=TERM_RED; 
				if( (strstr(msg, "was slain by")!=NULL)) a=TERM_YELLOW;
				if( (strstr(msg, "rises from the dead")!=NULL)) a=TERM_ORANGE;
				if( (strstr(msg, "was killed by")!=NULL)) a=TERM_RED;
			};

			if(a != TERM_WHITE) {

				/* Dump the message on the appropriate line */
				Term_putstr(0, (h - 1) - i, -1, a, msg);

				/* Cursor */
				Term_locate(&x, &y);

				/* Clear to end of line */
				Term_erase(x, y, 255);
			
				i++;
			};
		}

                /* Fresh */
                Term_fresh();

		/* Restore */
		Term_activate(oldt);
	}

}

/*
 * Hack -- pass color info around this file
 */
static byte likert_color = TERM_WHITE;


/*
 * Returns a "rating" of x depending on y
 */
static cptr likert(int x, int y)
{
	/* Paranoia */
	if (y <= 0) y = 1;

	/* Negative values */
	if (x < 0)
	{
		likert_color = TERM_RED;
		return ("Very Bad");
	}

	/* Analyze the value */
	switch ((x / y))
	{
		case 0:
		case 1:
		{
			likert_color = TERM_RED;
			return ("Bad");
		}
		case 2:
		{
			likert_color = TERM_RED;
			return ("Poor");
		}
		case 3:
		case 4:
		{
			likert_color = TERM_YELLOW;
			return ("Fair");
		}
		case 5:
		{
			likert_color = TERM_YELLOW;
			return ("Good");
		}
		case 6:
		{
			likert_color = TERM_YELLOW;
			return ("Very Good");
		}
		case 7:
		case 8:
		{
			likert_color = TERM_L_GREEN;
			return ("Excellent");
		}
		case 9:
		case 10:
		case 11:
		case 12:
		case 13:
		{
			likert_color = TERM_L_GREEN;
			return ("Superb");
		}
		case 14:
		case 15:
		case 16:
		case 17:
		{
			likert_color = TERM_L_GREEN;
			return ("Heroic");
		}
		default:
		{
			likert_color = TERM_L_GREEN;
			return ("Legendary");
		}
	}
}

/*
 
/*
 * Obtain the "flags" for an item
 */
static void object_flags_aux(const object_type *o_ptr, u32b *f1, u32b *f2, u32b *f3)
{
	object_kind *k_ptr = &k_info[o_ptr->k_idx];

	/* Base object */
	(*f1) = k_ptr->flags1;
	(*f2) = k_ptr->flags2;
	(*f3) = k_ptr->flags3;

	/* Artifact */
	if (o_ptr->name1)
	{
		artifact_type *a_ptr = &a_info[o_ptr->name1];

		(*f1) = a_ptr->flags1;
		(*f2) = a_ptr->flags2;
		(*f3) = a_ptr->flags3;
	}

	/* Ego-item */
	if (o_ptr->name2)
	{
		ego_item_type *e_ptr = &e_info[o_ptr->name2];

		(*f1) |= e_ptr->flags1;
		(*f2) |= e_ptr->flags2;
		(*f3) |= e_ptr->flags3;
	}

	/* Extra powers */
	switch (o_ptr->xtra1)
	{
		case EGO_XTRA_SUSTAIN:
		{
			/* Choose a sustain */
			switch (o_ptr->xtra2 % 6)
			{
				case 0: (*f2) |= TR2_SUST_STR; break;
				case 1: (*f2) |= TR2_SUST_INT; break;
				case 2: (*f2) |= TR2_SUST_WIS; break;
				case 3: (*f2) |= TR2_SUST_DEX; break;
				case 4: (*f2) |= TR2_SUST_CON; break;
				case 5: (*f2) |= TR2_SUST_CHR; break;
			}

			break;
		}

		case EGO_XTRA_POWER:
		{
			/* Choose a power */
			switch (o_ptr->xtra2 % 9)
			{
				case 0: (*f2) |= TR2_RES_BLIND; break;
				case 1: (*f2) |= TR2_RES_CONF; break;
				case 2: (*f2) |= TR2_RES_SOUND; break;
				case 3: (*f2) |= TR2_RES_SHARDS; break;
				case 4: (*f2) |= TR2_RES_NETHER; break;
				case 5: (*f2) |= TR2_RES_NEXUS; break;
				case 6: (*f2) |= TR2_RES_CHAOS; break;
				case 7: (*f2) |= TR2_RES_DISEN; break;
				case 8: (*f2) |= TR2_RES_POIS; break;
			}

			break;
		}

		case EGO_XTRA_ABILITY:
		{
			/* Choose an ability */
			switch (o_ptr->xtra2 % 8)
			{
				case 0: (*f3) |= TR3_FEATHER; break;
				case 1: (*f3) |= TR3_LITE; break;
				case 2: (*f3) |= TR3_SEE_INVIS; break;
				case 3: (*f3) |= TR3_TELEPATHY; break;
				case 4: (*f3) |= TR3_SLOW_DIGEST; break;
				case 5: (*f3) |= TR3_REGEN; break;
				case 6: (*f2) |= TR2_FREE_ACT; break;
				case 7: (*f2) |= TR2_HOLD_LIFE; break;
			}

			break;
		}
	}
}


/*
 * Obtain the "flags" for an item
 */
void object_flags(const object_type *o_ptr, u32b *f1, u32b *f2, u32b *f3)
{
	object_flags_aux(o_ptr, f1, f2, f3);
}


/*
 * Obtain the "flags" for an item which are known to the player
 */
void object_flags_known(const object_type *o_ptr, u32b *f1, u32b *f2, u32b *f3)
{
	object_flags_aux(o_ptr, f1, f2, f3);
}

/*
 * Obtain the "flags" for the player as if he was an item
 */
void player_flags(u32b *f1, u32b *f2, u32b *f3)
{
	/* Clear */
	(*f1) = (*f2) = (*f3) = 0L;
 // DAD
	
	/* Add racial flags */
	/*(*f1) |= race_info[race].flags1;
	(*f2) |= race_info[race].flags2;
	(*f3) |= race_info[race].flags3;

	if (class_info[class].flags & CF_BRAVERY_30)
	{
		if (p_ptr->lev >= 30) (*f2) |= (TR2_RES_FEAR);
	}*/
}

/*
 * Equippy chars
 */


static void display_player_equippy(int y, int x)
{
	int i;

	byte a;
	char c;

	object_type *o_ptr;


	/* Dump equippy chars */
	for (i = INVEN_WIELD; i < INVEN_TOTAL; ++i)
	{
		/* Object */
		o_ptr = &inventory[i];

		/* Skip empty objects */
		a = 0;
		c = ' ';

		if (o_ptr->k_idx) 
		{
			/* Get attr/char for display */
			a = object_attr(o_ptr);
			c = object_char(o_ptr);
		}

		/* Dump */
		Term_putch(x+i-INVEN_WIELD, y, a, c);
	}
}

void prt_equip(void)
{
	display_player_equippy(ROW_EQUIP, COL_EQUIP);
}

/*
 * Hack -- see below
 */
static const byte display_player_flag_set[4] =
{
	2,
	2,
	3,
	1
};

/*
 * Hack -- see below
 */
static const u32b display_player_flag_head[4] =
{
	TR2_RES_ACID,
	TR2_RES_BLIND,
	TR3_SLOW_DIGEST,
	TR1_STEALTH
};

/*
 * Hack -- see below
 */
static cptr display_player_flag_names[4][8] =
{
	{
		" Acid:",	/* TR2_RES_ACID */
		" Elec:",	/* TR2_RES_ELEC */
		" Fire:",	/* TR2_RES_FIRE */
		" Cold:",	/* TR2_RES_COLD */
		" Pois:",	/* TR2_RES_POIS */
		" Fear:",	/* TR2_RES_FEAR */
		" Lite:",	/* TR2_RES_LITE */
		" Dark:"	/* TR2_RES_DARK */
	},

	{
		"Blind:",	/* TR2_RES_BLIND */
		"Confu:",	/* TR2_RES_CONFU */
		"Sound:",	/* TR2_RES_SOUND */
		"Shard:",	/* TR2_RES_SHARD */
		"Nethr:",	/* TR2_RES_NETHR */
		"Nexus:",	/* TR2_RES_NEXUS */
		"Chaos:",	/* TR2_RES_CHAOS */
		"Disen:"	/* TR2_RES_DISEN */
	},

	{
		"Feath:",	/* TR3_FEATHER */
		"PLite:",	/* TR3_LITE */
		"Invis:",	/* TR3_SEE_INVIS */
		"Telep:",	/* TR3_TELEPATHY */
		"S.Dig:",	/* TR3_SLOW_DIGEST */
		"Regen:",	/* TR3_REGEN */
		"FrAct:",	/* TR3_FREE_ACT */
		"HLife:"	/* TR3_HOLD_LIFE */
	},

	{
		"Stea.:",	/* TR1_STEALTH */
		"Sear.:",	/* TR1_SEARCH */
		"Infra:",	/* TR1_INFRA */
		"Tunn.:",	/* TR1_TUNNEL */
		"Speed:",	/* TR1_SPEED */
		"Blows:",	/* TR1_BLOWS */
		"Might:",	/* TR1_MIGHT */
		"Shots:"	/* TR1_SHOTS */
	}
};


static byte flag2table(int table, int f1, int f2, int f3)
{
	byte res;

	res = 0;
	switch(table)
	{
		case 1:
			res = (f2 & 0x00FF0000) >> 16;
			break;
		case 2:
			res = (f2 & 0xFF000000) >> 24;
			break;
		case 3:				       		
			res = (f3 & 0x0003F000) >> 12;
			res |= (f2 & 0x0000C000) >> 8;
			break;
		case 4:
			res = (f1 & 0x00003F00) >> 8;
			res |= (f3 & 0x000C0000) >> 12;

			break;
	}

	return res;
}

/*
 * Special display, part 1
 */
static void display_player_flag_info(void)
{
	int x, y, i, n;

	int row, col;

	int set;
	u32b head;
	u32b flag;
	cptr name;
	byte tblflag;

	u32b f[4];


	/* Four columns */
	for (x = 0; x < 4; x++)
	{
		/* Reset */
		row = 11;
		col = 20 * x;

		/* Extract set */
		set = display_player_flag_set[x];

		/* Extract head */
		head = display_player_flag_head[x];

		/* Header */
		c_put_str(TERM_WHITE, "abcdefghijkl@", row++, col+6);

		/* Eight rows */
		for (y = 0; y < 8; y++)
		{
			/* Extract flag */
			flag = (head << y);

			/* Extract name */
			name = display_player_flag_names[x][y];

			/* Header */
			c_put_str(TERM_WHITE, name, row, col);

			/* Check equipment */
			for (n = 6, i = INVEN_WIELD; i < INVEN_TOTAL; ++i, ++n)
			{
				byte attr = TERM_SLATE;

				object_type *o_ptr;
				
				/* Object */
				o_ptr = &inventory[i];

				/* Known flags */
				object_flags_known(o_ptr, &f[1], &f[2], &f[3]);

				/* Color columns by parity */
				if (i % 2) attr = TERM_L_WHITE;

				/* Non-existant objects */
				if (!o_ptr->k_idx) 
				{
					attr = TERM_L_DARK;
					tblflag = 0;
				}
				else
					tblflag = flag2table(x + 1, f[1], f[2], f[3]);

				/* Hack -- Check immunities */
				if ((x == 0) && (y < 4) &&
				    (f[set] & ((TR2_IM_ACID) << y)) &&
					o_ptr->k_idx)
				{
					c_put_str(TERM_WHITE, "*", row, col+n);
				}

				/* Check flags */
				else if (tblflag & (1 << y))
				{
					c_put_str(TERM_WHITE, "+", row, col+n);
				}
				/* Default */
				else
				{
					c_put_str(attr, ".", row, col+n);
				}
			}

			/* Player flags */
			player_flags(&f[1], &f[2], &f[3]);

			/* Default */
			c_put_str(TERM_SLATE, ".", row, col+n);

			/* Hack -- Check immunities */
			if ((x == 0) && (y < 4) &&
			    (f[set] & ((TR2_IM_ACID) << y)))
			{
				c_put_str(TERM_WHITE, "*", row, col+n);
			}

			/* Check flags */
			else if (f[set] & flag) c_put_str(TERM_WHITE, "+", row, col+n);

			/* Advance */
			row++;
		}

		/* Footer */
		c_put_str(TERM_WHITE, "abcdefghijkl@", row++, col+6);

		/* Equippy */
		display_player_equippy(row++, col+6);
	}
}


/*
 * Special display, part 2a
 */
static void display_player_misc_info(void)
{
	cptr p;

	char buf[80];


	/* Name */
	put_str("Name", 2, 1);
	c_put_str(TERM_L_BLUE, nick, 2, 8);


	/* Sex */
	put_str("Sex", 3, 1);
	c_put_str(TERM_L_BLUE, (p_ptr->male ? "Male" : "Female"), 3, 8);

	/* Race */
	put_str("Race", 4, 1);
	c_put_str(TERM_L_BLUE, race_info[race].title, 4, 8);


	/* Class */
	put_str("Class", 5, 1);
	c_put_str(TERM_L_BLUE, class_info[class].title, 5, 8);


	/* Title */
	put_str("Title", 6, 1);

	if (p_ptr->total_winner || (p_ptr->lev > PY_MAX_LEVEL))
	{
		p = "***WINNER***";
	}
	else
		p = p_ptr->title;

	/* Dump it */
	c_put_str(TERM_L_BLUE, p, 6, 8);


	/* Hit Points */
	put_str("HP", 7, 1);
	strnfmt(buf, sizeof(buf), "%d/%d", p_ptr->chp, p_ptr->mhp);
	c_put_str(TERM_L_BLUE, buf, 7, 8);


	/* Spell Points */
	put_str("SP", 8, 1);
	strnfmt(buf, sizeof(buf), "%d/%d", p_ptr->csp, p_ptr->msp);
	c_put_str(TERM_L_BLUE, buf, 8, 8);
}


/*
 * Special display, part 2b
 */
static void display_player_stat_info(void)
{
	int i, row, col;

	char buf[80];


	/* Row */
	row = 3;

	/* Column */
	col = 42;

	/* Print out the labels for the columns */
	c_put_str(TERM_WHITE, "  Self", row-1, col+5);
	c_put_str(TERM_WHITE, " RB", row-1, col+12);
	c_put_str(TERM_WHITE, " CB", row-1, col+16);
	c_put_str(TERM_WHITE, " EB", row-1, col+20);
	c_put_str(TERM_WHITE, "  Best", row-1, col+24);

	/* Display the stats */
	for (i = 0; i < 6; i++) 
	{
		/* Reduced */
		if (p_ptr->stat_use[i] < p_ptr->stat_top[i])
		{
			/* Use lowercase stat name */
			put_str(stat_names_reduced[i], row+i, col);
		}

		/* Normal */
		else
		{
			/* Assume uppercase stat name */
			put_str(stat_names[i], row+i, col);
		}

		/* Indicate natural maximum */
		if (p_ptr->stat_max[i] == 18+100)
		{
			put_str("!", row+i, col+3);
		}

		/* Internal "natural" maximum value */ 
		cnv_stat(p_ptr->stat_max[i], buf);
		c_put_str(TERM_L_GREEN, buf, row+i, col+5);

		/* Race Bonus */
		strnfmt(buf, sizeof(buf), "%+3d", race_info[race].r_adj[i]);
		c_put_str(TERM_L_BLUE, buf, row+i, col+12);

		/* Class Bonus */
		strnfmt(buf, sizeof(buf), "%+3d", class_info[class].c_adj[i]);
		c_put_str(TERM_L_BLUE, buf, row+i, col+16);

		/* Equipment Bonus */		
		strnfmt(buf, sizeof(buf), "%+3d", p_ptr->stat_add[i]);
		c_put_str(TERM_L_BLUE, buf, row+i, col+20);

		/* Resulting "modified" maximum value */
		cnv_stat(p_ptr->stat_top[i], buf);
		c_put_str(TERM_L_GREEN, buf, row+i, col+24);

		/* Only display stat_use if not maximal */
		if (p_ptr->stat_use[i] < p_ptr->stat_top[i])
		{
			cnv_stat(p_ptr->stat_use[i], buf);
			c_put_str(TERM_YELLOW, buf, row+i, col+31);
		}
	}
}


/*
 * Special display, part 2c
 *
 * How to print out the modifications and sustains.
 * Positive mods with no sustain will be light green.
 * Positive mods with a sustain will be dark green.
 * Sustains (with no modification) will be a dark green 's'.
 * Negative mods (from a curse) will be red.
 * Huge mods (>9), like from MICoMorgoth, will be a '*'
 * No mod, no sustain, will be a slate '.'
 */
static void display_player_sust_info(void)
{
	int i, row, col, stat;

	object_type *o_ptr;
	u32b f1, f2, f3;
	u32b ignore_f2, ignore_f3;
	s32b pval;

	byte a;
	char c;


	/* Row */
	row = 3;

	/* Column */
	col = 26;

	/* Header */
	c_put_str(TERM_WHITE, "abcdefghijkl@", row-1, col);

	/* Process equipment */
	for (i = INVEN_WIELD; i < INVEN_TOTAL; ++i)
	{
		/* Get the object */
		o_ptr = &inventory[i];

		/* Get the "known" flags */
		object_flags_known(o_ptr, &f1, &f2, &f3);

		/* Hack -- assume stat modifiers are known */
		object_flags(o_ptr, &f1, &ignore_f2, &ignore_f3);

		/* Initialize color based of sign of pval. */
		for (stat = 0; stat < 6; stat++) 
		{
			/* Default */
			a = TERM_SLATE;
			c = '.';
			
			/* Boost */
			if ((f1 & (1<<stat)) && o_ptr->k_idx)
			{
				/* Default */
				c = '*';

				/* pval from Server */
				pval = o_ptr->pval;

				/* Aditional pval info for items */
				if (k_info[o_ptr->k_idx].flags1 & (TR1_CHR | TR1_DEX | TR1_CON | TR1_STR | TR1_WIS | TR1_INT))
					pval += o_ptr->bpval;

				/* Good */
				if (pval > 0)
				{
					/* Good */
					a = TERM_L_GREEN;

					/* Label boost */
					if (pval < 10) c = I2D(pval);
				}

				/* Bad */
				if (pval < 0)
				{
					/* Bad */
					a = TERM_RED;

					/* Label boost */
					if (pval > -10) c = I2D(-(pval));
				}
			}

			/* Sustain */
			if (f2 & (1<<stat) && o_ptr->k_idx)
			{
				/* Dark green */
				a = TERM_GREEN;

				/* Convert '.' to 's' */
				if (c == '.') c = 's';
			}

			/* Dump proper character */
			Term_putch(col, row+stat, a, c);
		}

		/* Advance */
		col++;
	}

	/* Player flags */
	player_flags(&f1, &f2, &f3);

	/* Check stats */
	for (stat = 0; stat < 6; ++stat) 
	{
		/* Default */
		a = TERM_SLATE;
		c = '.';

		/* Sustain */
		if (f2 & (1<<stat) && o_ptr->k_idx)
		{
			/* Dark green "s" */
			a = TERM_GREEN;
			c = 's';
		}

		/* Dump */
		Term_putch(col, row+stat, a, c);
	}

	/* Column */
	col = 26;

	/* Footer */
	c_put_str(TERM_WHITE, "abcdefghijkl@", row+6, col);

	/* Equippy */
	display_player_equippy(row+7, col);
}

/*
 * Prints some "extra" information on the screen.
 *
 * Space includes rows 3-9 cols 24-79
 * Space includes rows 10-17 cols 1-79
 * Space includes rows 19-22 cols 1-79
 */
static void display_player_xtra_info(void)
{
	int i;
	int col;
	int hit, dam;
	int base, plus;
	int tmp;
	int xthn, xthb, xfos, xsrh;
	int xdis, xdev, xsav, xstl;
	byte likert_attr;

	object_type *o_ptr;

	cptr desc;

	char buf[160];
	char depths[32];


	/* Upper middle */
	col = 26;


	/* Age */
	Term_putstr(col, 3, -1, TERM_WHITE, "Age");
	Term_putstr(col+9, 3, -1, TERM_L_BLUE, format("%4d", (int)p_ptr->age));

	/* Height */
	Term_putstr(col, 4, -1, TERM_WHITE, "Height");
	Term_putstr(col+9, 4, -1, TERM_L_BLUE, format("%4d", (int)p_ptr->ht));

	/* Weight */
	Term_putstr(col, 5, -1, TERM_WHITE, "Weight");
	Term_putstr(col+9, 5, -1, TERM_L_BLUE, format("%4d", (int)p_ptr->wt));

	/* Status */
	Term_putstr(col, 6, -1, TERM_WHITE, "Status");
	Term_putstr(col+9, 6, -1, TERM_L_BLUE, format("%4d", (int)p_ptr->sc));

	/* Left */
	col = 1;

	/* Level */
	Term_putstr(col, 10, -1, TERM_WHITE, "Level");
	Term_putstr(col+8, 10, -1, TERM_L_GREEN,
		            format("%10d", p_ptr->lev));

	/* Current Experience */
	Term_putstr(col, 11, -1, TERM_WHITE, "Cur Exp");
	if (p_ptr->exp >= p_ptr->max_exp)
	{
		Term_putstr(col+8, 11, -1, TERM_L_GREEN,
		            format("%10ld", p_ptr->exp));
	}
	else
	{
		Term_putstr(col+8, 11, -1, TERM_YELLOW,
		            format("%10ld", p_ptr->exp));
	}


	/* Maximum Experience */
	Term_putstr(col, 12, -1, TERM_WHITE, "Max Exp");
	Term_putstr(col+8, 12, -1, TERM_L_GREEN,
	            format("%10ld", p_ptr->max_exp));


	/* Advance Experience */
	Term_putstr(col, 13, -1, TERM_WHITE, "Adv Exp");
	if (p_ptr->lev < PY_MAX_LEVEL)
	{
		s32b advance = exp_adv;
		Term_putstr(col+8, 13, -1, TERM_L_GREEN,
		            format("%10ld", advance));
	}
	else
	{
		Term_putstr(col+8, 13, -1, TERM_L_GREEN,
		            format("%10s", "********"));
	}

	/* Maximum depth */
	Term_putstr(col, 14, -1, TERM_WHITE, "MaxDepth");

	if (!p_ptr->max_dlv)
	{
		strncpy(depths, "Town", sizeof(depths));
	}
	else if (depth_in_feet)
	{
		strnfmt(depths, sizeof(depths), "%d ft", p_ptr->max_dlv * 50);
	}
	else
	{
		strnfmt(depths, sizeof(depths), "Lev %d", p_ptr->max_dlv);
	}

	Term_putstr(col+8, 14, -1, TERM_L_GREEN,
	            format("%10s", depths));


	/* Gold */
	Term_putstr(col, 15, -1, TERM_WHITE, "Gold");
	Term_putstr(col+8, 15, -1, TERM_L_GREEN,
	            format("%10ld", p_ptr->au));

	/* Burden */
	for(p_ptr->total_weight = 0, i = INVEN_WIELD; 
		i < INVEN_TOTAL; 
		p_ptr->total_weight += inventory[i++].weight);
	
	strnfmt(buf, sizeof(buf), "%ld.%ld lbs",
	        p_ptr->total_weight / 10L,
	        p_ptr->total_weight % 10L);
	Term_putstr(col, 16, -1, TERM_WHITE, "Burden");
	Term_putstr(col+8, 16, -1, TERM_L_GREEN,
	            format("%10s", buf));

	/* Speed (without temporary effects) */
	Term_putstr(col, 17, -1, TERM_WHITE, "Speed");
	tmp = p_ptr->pspeed;

	if (tmp != 0)
	{
		Term_putstr(col+8, 17, -1, TERM_L_GREEN,
		            format("%+10d", tmp));
	}
	else
	{
		Term_putstr(col+12, 17, -1, TERM_L_GREEN, "Normal");
	}


	/* Middle */
	col = 26;

	/* Armor */
	base = p_ptr->dis_ac;
	plus = p_ptr->dis_to_a;

	/* Total Armor */
	strnfmt(buf, sizeof(buf), "[%d,%+d]", base, plus);
	Term_putstr(col, 10, -1, TERM_WHITE, "Armor");
	Term_putstr(col+5, 10, -1, TERM_L_BLUE, format("%13s", buf));

	/* Base skill */
	o_ptr = &inventory[INVEN_WIELD];
	hit = p_ptr->dis_to_h - o_ptr->to_h;
	dam = p_ptr->dis_to_d - o_ptr->to_d;

	/* Basic fighting */
	strnfmt(buf, sizeof(buf), "(%+d,%+d)", hit, dam);
	Term_putstr(col, 11, -1, TERM_WHITE, "Fight");
	Term_putstr(col+5, 11, -1, TERM_L_BLUE, format("%13s", buf));

	/* Melee weapon */
	o_ptr = &inventory[INVEN_WIELD];

	/* Melee attacks */
	strnfmt(buf, sizeof(buf), "(%+d,%+d)", hit + o_ptr->to_h, dam + o_ptr->to_d);
	Term_putstr(col, 12, -1, TERM_WHITE, "Melee");
	Term_putstr(col+5, 12, -1, TERM_L_BLUE, format("%13s", buf));

	/* Range weapon */
	o_ptr = &inventory[INVEN_BOW];

	/* Range attacks */
	strnfmt(buf, sizeof(buf), "(%+d,%+d)", hit + o_ptr->to_h, 0 + o_ptr->to_d);
	Term_putstr(col, 13, -1, TERM_WHITE, "Shoot");
	Term_putstr(col+5, 13, -1, TERM_L_BLUE, format("%13s", buf));

	/* Blows */
	strnfmt(buf, sizeof(buf), "%d/turn", p_ptr->num_blow);
	Term_putstr(col, 14, -1, TERM_WHITE, "Blows");
	Term_putstr(col+5, 14, -1, TERM_L_BLUE, format("%13s", buf));

	/* Shots */
	strnfmt(buf, sizeof(buf), "%d/turn", p_ptr->num_fire);
	Term_putstr(col, 15, -1, TERM_WHITE, "Shots");
	Term_putstr(col+5, 15, -1, TERM_L_BLUE, format("%13s", buf));


	/* Infra */
	strnfmt(buf, sizeof(buf), "%d ft", p_ptr->see_infra * 10);
	Term_putstr(col, 17, -1, TERM_WHITE, "Infra");
	Term_putstr(col+5, 17, -1, TERM_L_BLUE, format("%13s", buf));


	/* Right */
	col = 49;


	/* Fighting Skill (with current weapon) */
	o_ptr = &inventory[INVEN_WIELD];
	tmp = p_ptr->to_h + o_ptr->to_h;
	xthn = p_ptr->skill_thn + (tmp * BTH_PLUS_ADJ);

	/* Shooting Skill (with current bow) */
	o_ptr = &inventory[INVEN_BOW];
	tmp = p_ptr->to_h + o_ptr->to_h;
	xthb = p_ptr->skill_thb + (tmp * BTH_PLUS_ADJ);

	/* Basic abilities */
	xdis = p_ptr->skill_dis;
	xdev = p_ptr->skill_dev;
	xsav = p_ptr->skill_sav;
	xstl = p_ptr->skill_stl;
	xsrh = p_ptr->skill_srh;
	xfos = p_ptr->skill_fos;

	put_str("Saving Throw", 10, col);
	desc = likert(p_ptr->skill_sav, 6);
	c_put_str(likert_color, format("%9s", desc), 10, col+14);

	put_str("Stealth", 11, col);
	desc = likert(p_ptr->skill_stl, 1);
	c_put_str(likert_color, format("%9s", desc), 11, col+14);

	put_str("Fighting", 12, col);
	desc = likert(p_ptr->skill_thn, 12);
	c_put_str(likert_color, format("%9s", desc), 12, col+14);

	put_str("Shooting", 13, col);
	desc = likert(p_ptr->skill_thb, 12);
	c_put_str(likert_color, format("%9s", desc), 13, col+14);

	put_str("Disarming", 14, col);
	desc = likert(p_ptr->skill_dis, 8);
	c_put_str(likert_color, format("%9s", desc), 14, col+14);

	put_str("Magic Device", 15, col);
	desc = likert(p_ptr->skill_dev, 6);
	c_put_str(likert_color, format("%9s", desc), 15, col+14);

	put_str("Perception", 16, col);
	desc = likert(p_ptr->skill_fos, 6);
	c_put_str(likert_color, format("%9s", desc), 16, col+14);

	put_str("Searching", 17, col);
	desc = likert(p_ptr->skill_srh, 6);
	c_put_str(likert_color, format("%9s", desc), 17, col+14);

	/* History */
	Term_gotoxy(1, 19);

	for (i = 0; i < 4; i++)
	{
		put_str(p_ptr->history[i], i + 19, 10);
	}
}

void display_player(int mode)
{
	/* Erase screen */
	clear_from(0);

	/* Misc info */
	display_player_misc_info();

	/* Stat info */
	display_player_stat_info();

	/* Special */
	if (mode)
	{
		/* Hack -- Level */
		put_str("Level", 9, 1);
		c_put_str(TERM_L_BLUE, format("%d", p_ptr->lev), 9, 8);

		/* Stat/Sustain flags */
		display_player_sust_info();

		/* Other flags */
		display_player_flag_info();
	}

	/* Standard */
	else
	{
		/* Extra info */
		display_player_xtra_info();
	}
}

#if 0 // DAD - Changing
void display_player(int hist)
{
	int i;
	char buf[80];
	cptr desc;

        /* Clear screen */
        Term_clear();

        /* Name, Sex, Race, Class */
        put_str("Name        :", 2, 1);
        put_str("Sex         :", 3, 1);
        put_str("Race        :", 4, 1);
        put_str("Class       :", 5, 1);

        c_put_str(TERM_L_BLUE, nick, 2, 15);
        c_put_str(TERM_L_BLUE, (p_ptr->male ? "Male" : "Female"), 3, 15);
        c_put_str(TERM_L_BLUE, race_info[race].title, 4, 15);
        c_put_str(TERM_L_BLUE, class_info[class].title, 5, 15);

        /* Age, Height, Weight, Social */
        prt_num("Age          ", (int)p_ptr->age, 2, 32, TERM_L_BLUE);
        prt_num("Height       ", (int)p_ptr->ht, 3, 32, TERM_L_BLUE);
        prt_num("Weight       ", (int)p_ptr->wt, 4, 32, TERM_L_BLUE);
        prt_num("Social Class ", (int)p_ptr->sc, 5, 32, TERM_L_BLUE);

        /* Display the stats */
        for (i = 0; i < 6; i++)
        {
                /* Special treatment of "injured" stats */
                if (p_ptr->stat_use[i] < p_ptr->stat_top[i])
                {
                        int value;

                        /* Use lowercase stat name */
                        put_str(stat_names_reduced[i], 2 + i, 61);

                        /* Get the current stat */
                        value = p_ptr->stat_use[i];

                        /* Obtain the current stat (modified) */
                        cnv_stat(value, buf);

                        /* Display the current stat (modified) */
                        c_put_str(TERM_YELLOW, buf, 2 + i, 66);

                        /* Acquire the max stat */
                        value = p_ptr->stat_top[i];

                        /* Obtain the maximum stat (modified) */
                        cnv_stat(value, buf);

                        /* Display the maximum stat (modified) */
                        c_put_str(TERM_L_GREEN, buf, 2 + i, 73);
                }

                /* Normal treatment of "normal" stats */
                else
                {
                        /* Assume uppercase stat name */
                        put_str(stat_names[i], 2 + i, 61);

                        /* Obtain the current stat (modified) */
                        cnv_stat(p_ptr->stat_use[i], buf);

                        /* Display the current stat (modified) */
                        c_put_str(TERM_L_GREEN, buf, 2 + i, 66);
                }
        }

	/* Check for history */
	if (hist)
	{
		put_str("(Character Background)", 15, 25);

		for (i = 0; i < 4; i++)
		{
			put_str(p_ptr->history[i], i + 16, 10);
		}
	}
	else
	{
		put_str("(Miscellaneous Abilities)", 15, 25);
		
		/* Display "skills" */
		put_str("Fighting    :", 16, 1);
		desc = likert(p_ptr->skill_thn, 12);
		c_put_str(likert_color, desc, 16, 15);

		put_str("Bows/Throw  :", 17, 1);
		desc = likert(p_ptr->skill_thb, 12);
		c_put_str(likert_color, desc, 17, 15);

		put_str("Saving Throw:", 18, 1);
		desc = likert(p_ptr->skill_sav, 6);
		c_put_str(likert_color, desc, 18, 15);

		put_str("Stealth     :", 19, 1);
		desc = likert(p_ptr->skill_stl, 1);
		c_put_str(likert_color, desc, 19, 15);


		put_str("Perception  :", 16, 28);
		desc = likert(p_ptr->skill_fos, 6);
		c_put_str(likert_color, desc, 16, 42);

		put_str("Searching   :", 17, 28);
		desc = likert(p_ptr->skill_srh, 6);
		c_put_str(likert_color, desc, 17, 42);

		put_str("Disarming   :", 18, 28);
		desc = likert(p_ptr->skill_dis, 8);
		c_put_str(likert_color, desc, 18, 42);

		put_str("Magic Device:", 19, 28);
		desc = likert(p_ptr->skill_dev, 6);
		c_put_str(likert_color, desc, 19, 42);


		put_str("Blows/Round:", 16, 55);
		put_str(format("%d", p_ptr->num_blow), 16, 69);

		put_str("Shots/Round:", 17, 55);
		put_str(format("%d", p_ptr->num_fire), 17, 69);

		put_str("Infra-Vision:", 19, 55);
		put_str(format("%d feet", p_ptr->see_infra * 10), 19, 69);
	}

        /* Dump the bonuses to hit/dam */
        prt_num("+ To Hit    ", p_ptr->dis_to_h, 9, 1, TERM_L_BLUE);
        prt_num("+ To Damage ", p_ptr->dis_to_d, 10, 1, TERM_L_BLUE);

        /* Dump the armor class bonus */
        prt_num("+ To AC     ", p_ptr->dis_to_a, 11, 1, TERM_L_BLUE);

        /* Dump the total armor class */
        prt_num("  Base AC   ", p_ptr->dis_ac, 12, 1, TERM_L_BLUE);

        prt_num("Level      ", (int)p_ptr->lev, 9, 28, TERM_L_GREEN);

        if (p_ptr->exp >= p_ptr->max_exp)
        {
                prt_lnum("Experience ", p_ptr->exp, 10, 28, TERM_L_GREEN);
        }
        else
        {
                prt_lnum("Experience ", p_ptr->exp, 10, 28, TERM_YELLOW);
        }

        prt_lnum("Max Exp    ", p_ptr->max_exp, 11, 28, TERM_L_GREEN);

        if (p_ptr->lev >= PY_MAX_LEVEL)
        {
                put_str("Exp to Adv.", 12, 28);
                c_put_str(TERM_L_GREEN, "    *****", 12, 28+11);
        }
        else
        {
                prt_lnum("Exp to Adv.", exp_adv, 12, 28, TERM_L_GREEN);
        }

        prt_lnum("Gold       ", p_ptr->au, 13, 28, TERM_L_GREEN);

        prt_num("Max Hit Points ", p_ptr->mhp, 9, 52, TERM_L_GREEN);

        if (p_ptr->chp >= p_ptr->mhp)
        {
                prt_num("Cur Hit Points ", p_ptr->chp, 10, 52, TERM_L_GREEN);
        }
        else if (p_ptr->chp > (p_ptr->mhp) / 10)
        {
                prt_num("Cur Hit Points ", p_ptr->chp, 10, 52, TERM_YELLOW);
        }
        else
        {
                prt_num("Cur Hit Points ", p_ptr->chp, 10, 52, TERM_RED);
        }

        prt_num("Max SP (Mana)  ", p_ptr->msp, 11, 52, TERM_L_GREEN);

        if (p_ptr->csp >= p_ptr->msp)
        {
                prt_num("Cur SP (Mana)  ", p_ptr->csp, 12, 52, TERM_L_GREEN);
        }
        else if (p_ptr->csp > (p_ptr->msp) / 10)
        {
                prt_num("Cur SP (Mana)  ", p_ptr->csp, 12, 52, TERM_YELLOW);
        }
        else
        {
                prt_num("Cur SP (Mana)  ", p_ptr->csp, 12, 52, TERM_RED);
        }
}
#endif
/*
 * Redraw any necessary windows
 */
void window_stuff(void)
{
	/* Window stuff */
	if (!p_ptr->window) return;

	/* Display inventory */
	if (p_ptr->window & PW_INVEN)
	{
		p_ptr->window &= ~(PW_INVEN);
		fix_inven();
	}

	/* Display equipment */
	if (p_ptr->window & PW_EQUIP)
	{
		p_ptr->window &= (~PW_EQUIP);
		fix_equip();
	}

	/* Display player */
	if (p_ptr->window & PW_PLAYER)
	{
		p_ptr->window &= (~PW_PLAYER);
		fix_player();
	}

	/* Display messages */
	if (p_ptr->window & PW_MESSAGE)
	{
		p_ptr->window &= (~PW_MESSAGE);
		fix_message();
	}
}

	
